Catch

Struct Catch 

pub struct Catch<'a, T>(/* private fields */);
A Catch<T> is a Deref/DerefMut wrapper around a T and a F: FnOnce(T) which calls that function on T in place of drop.

This allows you to provide an owned static value with no indirection or locking to somewhere that needs one, and recover the value automatically by a means of your choosing afterwards.

Implementations

Source

impl<'a, T: Send + 'a> Catch<'a, T>

pub fn new(this: T, catch: Box<dyn FnOnce(T) + Send + 'a>) -> Self

Create a new Catch, which will call the function catch on this when it is dropped.

pub fn disarm(Catch: Self) -> (T, Box<dyn FnOnce(T) + 'a>)

Disarm a Catch by unwrapping it into its original T and catch: impl FnOnce(T). If you drop the T now, it will be gone forever – but if you wrap the two together again using Catch::new it will again be caught on drop.

You might use this function if you need owned access to the T itself, but beware: if you’re expecting that the catch function is always called, you need to manually ensure that the Catch is re-assembled or the function called on every code path you care about.

pub fn sync(this: T) -> (Self, impl Fn() -> Option<T>)

Convenience method for the common case when you want to use Catch in a synchronous context, using an internal mutex to hold the caught item.

The returned closure will yield None until the Catch is dropped, and then will yield Some(T) exactly once, before yielding None again forever.

pub fn future(this: T) -> (Self, impl Future<Output = Option<T>> + Send + 'a)

Convenience method for the common case when you want to use Catch in an asynchronous context, using an internal tokio::sync::oneshot to hold the caught item.

The returned future will pend until this is dropped, whereupon it will immediately yield Some(T). If the Catch is disarmed and the catch function dropped, the future will yield None.